/* 
xkrs teletext display program.
NewsFleX is registered Copyright (C) <Jan Mourer>
email: jan@panteltje.demon.nl
snail mail: 
J.Mourer
Monnikebildtdijk 2
9078VE Oude Bildtzijl
Holland

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*ir remote driver for ir led, for samsung videoa recorder sv31xk*/

#include "sys_head.h"


#define IR_DATA_LENGTH 500000
#define PAR_PORT 0x378

	
qpro_vcr(int channel, int onhour, int onminute,\
int offhour, int offminute, int onday)
/*program vcr via ir remote with qpro system (samsung sv31xk, vcr must be on)*/
{
char temp[20];

if(!ir_remote_send("qpro"))return(0);/*select qpro menu*/

if(!ir_send_two_digits(channel))return(0);
if(!ir_send_two_digits(onhour))return(0);
if(!ir_send_two_digits(onminute))return(0);
if(!ir_send_two_digits(offhour))return(0);
if(!ir_send_two_digits(offminute))return(0);
if(!ir_send_two_digits(onday))return(0);

if(!ir_remote_send("qpro"))return(0);/*leave program menu*/
if(!ir_remote_send("qpro"))return(0);/*leave qpro menu*/
return(1);
}

ir_send_two_digits(int value)/*send a 2 digit value to the vcr*/
{
char temp[20];
char command[3];

sprintf(temp,"%02d", value);
command[1] = 0;/*strimg termination*/
command[0] = temp[0];
if(!ir_remote_send(command))return(0);/*send tenths*/
command[0] = temp[1];
if(!ir_remote_send(command))return(0);/*send units*/
return(1);
}

ir_remote_send(char command[])/*send a infrared remote commnd*/
{
/*
If the 486 has more then 16 Mb ram, the L1 (internal) cache can only
cache 16 Mb.
L2 cache is to slow if any wait states.
The stack would be high in memory and possibly not cached.
This would run to slow.
Using a static just seems to solve this (no guarantee).
(I upgraded from 12 Mb to 24 Mb, and every thing slowed down !)
*/
static char irdata[IR_DATA_LENGTH];
register int a;
register char data_read;
register prev_data;
unsigned char one, zero;
long li,ld;
char command_file[40];
char path[40];
FILE *ircmdfile;
int c;
char split;
int i;

get_path_name("ir_remote_command_files_dir", path);
sprintf(command_file, "%s%s.irc", path, command);
ircmdfile = fopen(command_file, "rb");
if(!ircmdfile)
	{
	printf("\nCannot open infrared remote command file %s\n", command_file);
	return(0);
	}	
li = 0;
while(1)
	{
	c = getc(ircmdfile);
	if(c == EOF)break;
	for(i = 0; i < 8; i ++)
		{
		split = c & 128;/*get bit 7, also note: c is an integer*/
		c <<= 1;/*shift left one position*/
		irdata[li] = split;/*0 or 128*/
		li++;
		if(li == IR_DATA_LENGTH)break;
		}
	if(li >= IR_DATA_LENGTH)break;/*array boundary*/
	}
fclose(ircmdfile);
data_read = inb(PAR_PORT);
one = data_read | 128;
zero = data_read & 127;
for(li = 0; li < IR_DATA_LENGTH; li++)
	{
	if(irdata[li])outb(one, PAR_PORT);
	else outb(zero, PAR_PORT);
	}
sdah();
sclh();
return(1);
}

ir_remote_learn()/*learn an infrared remote command*/
{
char irdata[IR_DATA_LENGTH];
register int a;
register char data_read;
unsigned char one, zero;
long li,ld;
FILE *ircmdfile;
char c;
char str[20];
char command_name[20];
char command_file[40];
char path[40];
unsigned char combined;
int i;

printf("\nsend IR commmand continuously, press any key\n");
while(!kbhit()){}/*wait for key*/
ld = 0;/*used as delay only, compiler removes constructions like a = a
/*repeated many times*/
for(li = 0; li < IR_DATA_LENGTH; li++)
	{
	a = inb(PAR_PORT + 1) & 8;/*bit 3 only*/
	ld++;/*for timing to get same timing as send loop*/
	ld--;
	ld++;
	irdata[li] = a;
	}
printf("\nIR data has been read, press any key to replay continuously\n");
a = getkey();
while(!kbhit()){}/*wait for key*/
a = getkey();
printf("\nsending IR data continuously, any key to stop");
data_read = inb(PAR_PORT);
one = data_read | 128;
zero = data_read & 127;
while(1)
	{
	if(kbhit())break;
	/*the following loop should not be cahnged for timing reasons*/
	for(li = 0; li < IR_DATA_LENGTH; li++)
		{
		if(irdata[li])outb(one, PAR_PORT);
		else outb(zero, PAR_PORT);
		}
	}	
/*now going to save in file*/
getkey();
while(1)/*ask for command name*/
	{
	printf("\nCommand name? ");
	fscanf(stdin, "%8s", command_name);
	printf("\n");
	get_path_name("ir_remote_command_files_dir", path);
	sprintf(command_file, "%s%s.irc", path, command_name);
	ircmdfile = fopen(command_file, "r");
	if(ircmdfile)
		{
		fclose(ircmdfile);
		printf("\nCommand file %s already exists, overwrite (y/n)? \n",\
		command_file);
		fscanf(stdin,"%1s", str);
		toupper(str[0]);
		if(str[0] == 'Y')break;/*from while asking command name*/
		printf("\nCancel data read (y/n)? ");
		fscanf(stdin, "%1s", str);
		toupper(str[0]);
		if(str[0] == 'N')return(0);/*error, loose data*/
		}
	else break;/*create*/
	}/*end while ask for command name*/
ircmdfile = fopen(command_file, "wb");
if(!ircmdfile)
	{
	printf("\nCannot open command file %s for write\n", command_name);
	return(0);/*error*/
	}
printf("\nsaving data");
li = 0;
while(1)
	{
	if(li >= IR_DATA_LENGTH)break;/*array boundary*/
	combined = 0;/*reset combined bits to 0*/
	for(i = 0; i < 8; i++)
		{
		c = irdata[li];/*get a bit*/
		li++;/*point to next element in array*/
		if(li == IR_DATA_LENGTH)break;
		combined <<= 1;/*shift left byte one position*/
		/*note << sets bit 0 to zero*/
		if(c)combined |= 1;/*set bit 0*/
		}/*end for each 8 bits*/
	putc(combined, ircmdfile);/*write a byte*/
	}/*end while all data in irdata array*/
fclose(ircmdfile);
printf("\nReady\n");
return(1);
}

test()
{
char irdata[IR_DATA_LENGTH];
register int a;
register char data_read;
unsigned char one, zero;
long li,ld;
printf("\nsend IR commmand continuously, press any key\n");
while(!kbhit()){}/*wait for key*/
ld = 0;/*used as delay only, compiler removes constructions like a = a
/*repeated many times*/
for(li = 0; li < IR_DATA_LENGTH; li++)
	{
	a = inb(PAR_PORT + 1) & 8;/*bit 3 only*/
	ld++;
	ld--;
	ld++;
	irdata[li] = a;
	}
printf("\nIR data has been read, press any key to replay\n");
a = getkey();
while(!kbhit()){}/*wait for key*/
a = getkey();
data_read = inb(PAR_PORT);
one = data_read | 128;
zero = data_read & 127;
while(1)
	{
	if(kbhit())break;
	for(li = 0; li < IR_DATA_LENGTH; li++)
		{
		if(irdata[li])outb(one, PAR_PORT);
		else outb(zero, PAR_PORT);
		}
	}	
printf("\nReady\n");
return(1);
}

ir_dly(d)
int d;
{
int i;
for(i=0;i<d;i++)
	{
	d=d;
	}
}

ir_out(int data)
{
unsigned char a;
a = inb(PAR_PORT);
if(data)a = a | 128;/*set bit 7, IR LED on*/
else a = a & 127;/*reset bit 7, IR LED off*/
outb(a, PAR_PORT);
}

ir_in()
{
char a;
int b;
a=inb(PAR_PORT + 1);
a=a & 8;/*bit 3 only (ERROR signal on pin 15)*/
b=a;
return(b);
}

