/* 
xkrs teletext display program.
NewsFleX is registered Copyright (C) <Jan Mourer>
email: jan@panteltje.demon.nl
snail mail: 
J.Mourer
Monnikebildtdijk 2
9078VE Oude Bildtzijl
Holland

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* xkrslib.c
/* library routines for krs.c
/* teletext using the SAA5246 chip */

#include "sys_head.h"

#include "xkrs.h"
#include "keys.h"
#include "s14font.h"
#include "txtfont.h"
#include "xglgraphi.h"
#include "display.h"


#define CONTINUOUS -1

#define GET_PAGE		0
#define GET_SUBPAGE		1
#define SELECTED		3

/* for macros in 'x' mode */
#define MACRO_OFF	0
#define MACRO_READ	1
#define MACRO_WRITE	2

/*commnds in x mode */
/* 0 through 9 for digits */
#define CLEAR_ENTRY				10
#define ERASE_PAGE				11
#define SAVE_PAGE				12
#define LOAD_PAGE				13
#define PROGRAM_VCR 			14
#define DELETE_LAST_TIMER		15
#define RESTORE_PREVIOUS_PAGE 	16
#define DISPLAY_GROUP_TOGGLE	17
#define ACQUISITION_BANK_TOGGLE	18
#define REVEAL_CONCEAL_TOGGLE	19
#define MACRO_A					20
#define MACRO_B					21
#define MACRO_C					22		   	
#define MACRO_D					23
#define MACRO_E					24
#define MACRO_F					25
#define MACRO_G					26
#define MACRO_H					27
#define MACRO_I					28

/* definition of teletext screen format in 'x' mode */
#define TXT_ROWS		24
#define TXT_COLUMNS		40

/*the arrays are needed because we need to know all parameters of an
/*acquisition circuit belonging to a specific page in the display all the time*/
/*for every acquisition circuit*/
static int _main_page[4];
static int _sub_page[4];
static int _acquisition_circuit[4];
static int _timed_page_flag[4];
static int _hold_page_flag[4];
static int _reveal_flag[8];/* there are 8 displays */
static int macro_flag;
static char present_macro_name[40];
static int hpos[10];
static int vpos[10];
static int x_initiated_flag;
static int last_active_bank;
static char inputstring[80];

static int oldpixels[10][10];
static int oldx, oldy;

static int mouse_click;

FILE *macrofile;


video_control(int request)
/* select video modes, using philips dtv1000 vga tv card */
{
static int mute_flag;
int data;
switch(request)
	{
	case TELETEXT:
		vidsel(2);/* only set saa mode to txt out*/
		data = inb(0x200);
		data &= 255 - 4;		/* reset bit 2, selects computer */
		outb(data, 0x200);
		break;
	case TV_PROGRAM:			/* selects video to txt decoder */
		vidsel(4);/* video switch + saa switch to video out*/
		data = inb(0x200);
		data |= 4;				/* set bit 2, selects output tv vga card */
		outb(data, 0x200);
		break;
	case AUX:	/* composite video from pal coder connected to saa output */
		vidsel(2);/* set saa mode to teletext only */
/*		seldisp(dspmem);/* select a saa dspmem */
		vidsel(1);/* select pal coder output */
		data = inb(0x200);
		data |= 4;				/* set bit 2, selects output tv vga card */
		outb(data, 0x200);
		break;
	case MUTE:			/* not implemented yet */
		mute_flag = 1 - mute_flag;
		break;
	}
return(1);
}

print_cursor_status(int x, int y, int dspmem_offsett)
/* print mouse position in txt page as column and row in display page 3 top
/* right corner */
{
int x_position, y_position;
int data_at_cursor;
int txt_vpos, txt_hpos;
int vert_page_position, hor_page_position;
int saa_mem;
int display;
char iic_data[3];
char temp[40];
#define UPPER 1
#define LOWER 2
#define LEFT 3
#define RIGHT 4
/*characterset s14font is 8 width and 9 hight*/
/*pages are 40 characters * 8 dots = 320 widht*/
/*the left oriented pages start at x = 0,
/*the right oriented pages start at x = 320.
/*the top oriented pages start at y = 0,
/*the bottom oriented pages start at y = 240.
*/
/*first calculate the row into a teletext page on screen*/
if(y > 239)/*lower 2 pages*/
	{
	y -= 240;
	vert_page_position = LOWER;
	}
else vert_page_position = UPPER;
/*calculate column into teletext page*/
if(x > 319)/*right 2 pages*/
	{
	x -= 320;
	hor_page_position = RIGHT;
	}
else hor_page_position = LEFT;
if(hor_page_position == LEFT && vert_page_position == UPPER)
	{
	display = 0;
	}
if(hor_page_position == RIGHT && vert_page_position == UPPER)
	{
	display = 1;
	}
if(hor_page_position == LEFT && vert_page_position == LOWER)
	{
	display = 2;
	}
if(hor_page_position == RIGHT && vert_page_position == LOWER)
	{
	display = 3;
	}
txt_vpos = y / charheight;/*first line is 0*/
txt_hpos = x / charwidth;/*first column is 0*/
/*we have screen (0 to 3) and x and y position in that screen*/
saa_mem = display + dspmem_offsett;
if(! pgetstr(txt_hpos, txt_vpos, saa_mem, 1, iic_data) )return(0);
data_at_cursor = iic_data[0];
sprintf(temp, "%02d %02d %03d", txt_hpos, txt_vpos, data_at_cursor);
x_position = 639 - 19 * charwidth + 1; /* for 9 characters plus spaces */
y_position = 0;
s3putmsg(x_position, y_position, WHITE, BLUE, temp, 1);
return(1);
}

print_macro_status(int macro_id, int macro_mode)
{
int i;
int macro_char;
int referenced_macro_char;
int macro_char_color;
int x_position;
int y_position;
char temp[80];
int fgcolor;
int bgcolor;
referenced_macro_char = macro_id - MACRO_A + 'A';
if(macro_mode == MACRO_OFF)macro_char_color = BLUE;
if(macro_mode == MACRO_WRITE)macro_char_color = RED;
if(macro_mode == MACRO_READ)macro_char_color = GREEN;
/* print the macro click bar ABCDEFGHI */
/*only the character referenced by macro_id changes background color*/
fgcolor = WHITE;
/* position of click bar */
y_position = 0;
x_position = 639 - 9 * charwidth; /* for 9 characters plus spaces */
temp[1] = 0;/* string termination */
i = 0;
while(1)
	{ 
	/* calculate the macro character to be printed */
	macro_char = i + 'A';
	/* calculate the back ground color */
	if(macro_char == referenced_macro_char)	bgcolor = macro_char_color;
	else bgcolor = BLUE;
	/* print macro character */
	temp[0] = macro_char;
	s3putmsg(x_position, y_position, fgcolor, bgcolor, temp, 1);
	/* calculate new x_position */
	x_position += charwidth;/* one space */
	if(x_position > 639)break;/* no more space */
	i++;
	}
return(1);
}

print_status()
/* print the status of the acquisition circuits in each display */
{
int i;
char temp[80];
for(i = 0; i < 4; i++)
	{
	if(_hold_page_flag[i])
		{
		sprintf(temp, "%03d/%03d HOLD",\
		_main_page[i],\
		_sub_page[i]);
		}
	else
		{
		sprintf(temp, "%03d/%03d ACQUIRE",\
		_main_page[i],\
		_sub_page[i]);
		}
	print_in_display(i, temp);
	}
return(1);
}

testdescriptor(char text[], char descriptor[], int dmem, int acf)
/*text is teletext read data,
/*descriptor is field from .krs file holding fund description,
/*spaces are represented by _ in descriptor*/
{
int i,j,k;
long l;
int errorflag;
char modified_descriptor[42];
char temp[42];
strcpy(modified_descriptor,descriptor);
/*modify _ for spaces, and to uppercase*/
for(i = 0; i < 40; i++)
	{
	if(descriptor[i] == '_')modified_descriptor[i] = ' ';/*also for 0 */
	else modified_descriptor[i] = toupper(descriptor[i]);
	if(descriptor[i] == 0)break;/*end of string*/
	}
tputstr(0,1,dmem,"                                        ");
												/*40 spaces, clear line*/
/*sprintf(temp,"%s '%s'",text,modified_descriptor);*/
sprintf(temp,"testing for text '%s'",modified_descriptor);
if(acf)printf("\ntesting for text '%s'",modified_descriptor);
tputstr(9,1,dmem,temp);
if(zindex(text, modified_descriptor) == -1)return(0);/*not in text*/
return(1);/*same*/
}

useralarm()/*signal user*/
{
int i;
long l;
for(i=0; i < 5; i++)/*beep sequence 3 times*/
	{
	beep(1,3000);
	delay(500);/* .5 second*/
	beep(1,4000);
	delay(500);/* .5 second*/
	}
return(1);/*not same*/
}

test_limit(int *dspmem, int *page, int *ttime, int *bank, int *acqcir,\
int *lstacq, int ctfl)
{
int file_id;
float fa,fb;
float flmin,flmax,fperc,fhyst,fprev,fhmax,fhmin;
char temp[80],ti[40],stock[10];
FILE *testfile;
FILE *limitsfile;
int a,b,c,d;
int i,j;
char fiwd[3], fimo[5];
int fida;
char si[20];
char beeper[5],check[5];
float trend;
int af,pg,sp,xpos,ypos,lenfie;
float fmaxk, fmink;
int lenfr,ps,pp;
int f;
int units,fraction;
float present_stockvalue;
int lic1,lic2;
char beeper_id[20];
char beeper_code[40];
int comport;
int beep_flag,beeper_in_use_flag;/*0 at start up*/
/*because of statics this function can be exit and re entered without re_beeping*/
static float old_stockvalue[LIMMAXFILES];
int move;
static int old_move[LIMMAXFILES];/*initialized to 0 at start*/
char filename[40];
char path[40];

vidsel(1);
printf("\nTesting limits of stockvalues\n");
get_path_name("acq_command_files_dir", path);
sprintf(filename, "%slimits.dat", path);
limitsfile = fopen(filename, "r");
if(! limitsfile)
	{
	printf("\nCannot open file %s\n", filename);
	return(0);
	}
/*first read in beeper and comport where modem is*/
a = fscanf(limitsfile,"beeper_id:%s comport:%d", beeper_id, &comport);
if(a == EOF)
	{
	printf(\
"\nFile format error: Cannot read beeper_id and comport from file %s\n",\
	filename);
	fclose(limitsfile);
	return(0);
	}
/*clear all display memories every time reading from limits.dat,
/*this ensures that new teletext data is read, and not repeatedly the
/*same values*/
for(*dspmem = 0; *dspmem < 4; (*dspmem)++)
	{
	tcls(*dspmem);
	/*request pages that will not be used, prevents SAA chip from hanging
	/*if same pages in different acq circuits*/
	/*only one of page 100 will be displayed (due to same fault)*/
	acqpag(100, 0, 0, *dspmem, 1 ,0);/*page, subpage, acqcir, time mode, !hold*/
	}
*dspmem=0;/*start here*/
while(1)/*for all entries in file limits.dat*/
	{
	printf("\n^Z for break\n");
	if(kbhit())/*key*/
		{
		if(getkey() == 26)/*^Z*/
			{
			printf("\nUser abort\n");
			fclose(limitsfile);
			return(0);/*user abort*/
			}
		}
	a=fscanf(limitsfile,\
	"%s	id:%d	max:%f	min:%f	perc:%f	hyst:%f	beeper:%s check:%s",\
	stock, &file_id, &flmax, &flmin, &fperc, &fhyst, beeper, check);
	if(a == EOF)
		{
		fclose(limitsfile);
		break;/*end of file on limits.dat*/
		}
	if(toupper(beeper[0]) == 'Y')
		{
		beep_flag=1;
		beeper_in_use_flag=1;/*beeper in use flag*/
		}
	else beep_flag=0;
	if(toupper(check[0]) != 'Y')continue;/*scip this entry in limits.dat*/
	get_path_name("stock_files_dir", path);
	sprintf(ti,"%s%s.krs", path, stock);
	/*open stock files*/
	testfile=fopen(ti,"r");/*testfile is fundname.krs, source*/
	if(!testfile)
		{
		printf("\nFile %s, cannot open for read",ti);
		return(0);
		}
	fscanf(testfile,"%s %s %d %s %f %d %d %d %d %d %d %f %f %d",\
	fiwd,fimo,&fida,si,&trend,&af,&pg,&sp,&xpos,&ypos,&lenfie,\
	&fmaxk,&fmink,&lenfr);/*get header from file*/
	while(1)/*read all entries from inputfile .krs*/
		{
		a=fscanf(testfile,"%s",temp);
		if(a == EOF)
			{
			break;
			}
		fb=atof(temp);
		if(fb > 0)fprev=fb;/*continuously updated, not in text or flag field*/
		}		
	fclose(testfile);/*have all we need*/
	if(fperc != 0)/*if percentage specified, else use data from limits.dat*/
		{
		flmax= fprev + (fperc * (fprev / 100.0) );
		flmin= fprev - (fperc * (fprev / 100.0) );
		}
	if(fhyst != 0)/*if hysteresis (treshhold) specified*/
		{
		fhmax= fprev + (fhyst * (fprev / 100.0) );
		fhmin= fprev - (fhyst * (fprev / 100.0) );
		}
	printf("\nChecking fund %s  ", stock);
	printf("prev %4.2f  ", fprev);
	printf("max %4.2f  min %4.2f  ", flmax, flmin);
	if(fperc != 0)printf("using perc %0.2f  ", fperc);
	if(beeper)printf("beeper on");
	printf("\n");
	/*display of teletext stockvalue from here*/
	vidsel(2);/*show what happens*/
	/*first test if requested page and subpage already
	/*assigned to an acq circuit (may not have arrived)*/
	a=0;/*acq ok flag*/
	for(*dspmem=0; *dspmem < 4; (*dspmem)++)/*test all dspmem*/
		{
		seldisp(*dspmem);/*visible*/
		getsupa(*dspmem,&ps,&pp);/*get subpage, page*/
		if((pp == pg) && (ps == sp))
			{
			a=1;/*acq ok flag*/
			break;/*req. one*/
			}
		}/*end for all dspmem*/
	if(!a)/*not assigned*/
		{
		*acqcir= *lstacq;/*assign oldest acq circuit*/
		*dspmem= *acqcir;
		seldisp(*dspmem);/*visible*/
		*page= pg;
		*ttime= sp;
		a=acqpag(*page, *ttime, *bank, *acqcir, 1, 0);
		*lstacq += 1;/*remember*/
		*lstacq %= 4;/*next one to be used*/
		}/*end if not  assigned*/
	if(a)/*already assigned or new assigned, acq ok*/
		{
		seldisp(*dspmem);/*visible*/
		clrstl(*dspmem);
		sprintf(temp,"testing %s  d%d", ti, *dspmem);/*ti=source.krs*/
		PF("\n%s",temp);
		a=waitpag(*dspmem, 35);
		/*a=0 is key or time out 35 minutes, 7 deep*/
		}
	if(a)/*acq was ok and page has arrived*/
		{
		lic1=MAXTRY;/*every MAXTRY tries reinitiate
					/*chip*/
					/*120 is appprox. 1 minute*/
		lic2=MAXINI;/*after MAXINI reinitiates of chip
					/*give up, next reopen of limits.dat will be followed
					/*by clear screen, including any errors*/
		f=1;/*first time flag for data read*/
					/*causes accumulation array clear*/
		while(1)/*keep trying to read value*/
			{
			a=getmoney(si,xpos,ypos,&units,&fraction,\
			*dspmem,lenfie,fmaxk,fmink,lenfr,f,ctfl, 1);/* range check is 1 */
			sprintf(temp,"left: rds:%d ini:%d",lic1,lic2);
			PF("\n%s",temp);
			lic1--;
			if(lic1 == 0)/*reinitiate chip*/
				{
				lic1=MAXTRY;
				lic2--;
				vidsel(1);
				vidsel(2);/*uses cctini(),
							/*since no data was read saa
							/*chip may hang due to bad
							/*signal or error on iicbus,
							/*this will reini chip*/
				}
			if(lic2 == 0)/*give up trying*/
				{
				a=0;/*error*/
				vidsel(1);
				printf("\nfund %s, cannot read teletext stockvalue", stock);
				beep(5,3000);/*signal user*/
				units=0;
				fraction=0;
				/*this wil send 00.00 for this fund to beeper (if enabled)*/
				break;/*from while try getmoney, with units=0 and fraction=0*/
				}
			f=0;/*if 1 array in getmoney is cleared*/
			if(a == 1)break;/*value read ok*/
			if(bdstat())/*key*/
				{
				if(getkey() == 26)/*^Z*/
					{
					printf("\nUser abort\n");/*^Z key*/
					return(0);
					}
				}
			}/*end while try getmoney*/
		}/*end acq was ok and page has arrived, or no data read  if(a)*/
	vidsel(1);
	/*value  now read ok*/
	sprintf(temp,"%d.%02d", units, fraction);/*combine*/
	sscanf(temp,"%f", &present_stockvalue);
	if(old_stockvalue[file_id] ==0)old_stockvalue[file_id]=fprev;/*start value*/
	/*report if limits reached*/
	if((present_stockvalue >= flmax) || (present_stockvalue <= flmin))
		{
		printf("\nSTOCKVALUE OUT OF RANGE\n");
/*		beep(2,3000);/*signal user*/ /*computer may be unattended,
						/*no repeated beeps*/
		}
	/*The beep conditions*/
	/*crossing max in positive direction*/
	move=0;
	if(present_stockvalue == 0)/*no data was read*/
		{
		move = -3;
		}
	if((present_stockvalue >= flmax) && (old_stockvalue[file_id] < flmax))
		{
		move= +2;
		printf("\ncrossing maximum in positive direction");
		}
	/*crossing min in negative direction*/
	if((present_stockvalue <= flmin) && (old_stockvalue[file_id] > flmin))
		{
		move= -2;
		printf("\ncrossing minimum in negative direction");
		}
	if(fhyst != 0)/*if hysteresis specified*/
		{
		/*crossing max hysteresis in negative direction*/
		if((present_stockvalue <= fhmax) && (old_stockvalue[file_id] > fhmax))
			{
			move= +1;
			printf("\ncrossing maximum hysteresis in negative direction");
			}
		/*crossing min hysteresis in positive direction*/
		if((present_stockvalue >= fhmin) && (old_stockvalue[file_id] < fhmin))
			{
			move= -1;
			printf("\ncrossing minimum hysteresis in positive direction");
			}
		}/*end if hyst != 0*/
	old_stockvalue[file_id]=present_stockvalue;/*remember*/
	printf("  now %4.2f  ", present_stockvalue);
	if(fhyst != 0)
		{
		printf("\nrepeat mode  hysteresis %2.2f%c", fhyst, '%');
		printf("\n%4.2f < %4.2f < *%4.2f* < %4.2f < %4.2f",\
		flmin, fhmin, present_stockvalue, fhmax, flmax);
		}
	printf("\n");
	if( (move) && (old_move[file_id] != move) )/*crossing limit,
										/*and not the same one again*/
		{
		if(beep_flag)
			{
			sprintf(beeper_code,"%d00%d%d", file_id, units, fraction);
			printf("sending code %s to beeper", beeper_code);

/*			if(!to_beeper(beeper_id, beeper_code, comport) ) */
			if(1 == 0)
				{
				printf("\nCannot send code %s to beeper using com%1d\n",\
				temp, comport);
				beep(5,3500);/*signal user*/
				return(0);
				}
			}/*end if beep flag*/

		else beep(5,3000);/*signal local user*/
		old_move[file_id] = move;/*remember*/
		}	
	printf("\n");
	}/*end while for all entries in file limits.dat*/
return(1);/*never here*/
}

calculate_teletext_page_and_position(int x, int y,\
	int *selected_item, int *selected_item_tail,\
	int *selected_display_page, int dspmem_offsett)
/* calculates value of page (*selected_item) and page (*selected_display_page)
/* in display (of 4 pages) mouse is pointing at,
/* dspmem_offsett is either 0, or 4 depending on display mode.
/* if dspmem_offsett is 0, data is read from saa display RAM 0 trough 3 for
/* *selected_display_page 0 trough 3,
/* if dspmem_offsett is 4, data is read from saa display RAM 4 trough 7 for
/* *selected_display_page 0 though 3.
/* checks for mouse click in special field bar and returns special field
/* commands in *selected_item.
/* returns -1 for *selected_item and *selected_display_page if invalid data.
*/
{
int txt_vpos, txt_hpos;
int vert_page_position, hor_page_position;
int saa_dsp_mem;
char temp[80];
#define UPPER 1
#define LOWER 2
#define LEFT 3
#define RIGHT 4
/*characterset s14font is 8 width and 9 hight*/
/*pages are 40 characters * 8 dots = 320 widht*/
/*the left oriented pages start at x = 0,
/*the right oriented pages start at x = 320.
/*the top oriented pages start at y = 0,
/*the bottom oriented pages start at y = 240.
*/
/*first calculate the row into a teletext page on screen*/
if(y > 239)/*lower 2 pages*/
	{
	y -= 240;
	vert_page_position = LOWER;
	}
else vert_page_position = UPPER;
/*calculate column into teletext page*/
if(x > 319)/*right 2 pages*/
	{
	x -= 320;
	hor_page_position = RIGHT;
	}
else hor_page_position = LEFT;
if(hor_page_position == LEFT && vert_page_position == UPPER)
	{
	*selected_display_page = 0;
	}
if(hor_page_position == RIGHT && vert_page_position == UPPER)
	{
	*selected_display_page = 1;
	}
if(hor_page_position == LEFT && vert_page_position == LOWER)
	{
	*selected_display_page = 2;
	}
if(hor_page_position == RIGHT && vert_page_position == LOWER)
	{
	*selected_display_page = 3;
	}
txt_vpos = y / charheight;/*first line is 0*/
txt_hpos = x / charwidth;/*first column is 0*/
/*check for valid x range*/
if( (txt_hpos < 0) || (txt_hpos > 39) )
	{
	*selected_item = -1;
	return(1);
	}
/*check for valid y range*/
if( (txt_vpos < 0) || (txt_vpos > 25) )
	{
	*selected_item = -1;
	return(1);
	}
/*we have screen (0 to 3) and x and y position in that screen*/
/*check for mouse click in special field bar*/
if(txt_vpos == 25)/*on special field bar*/
	{
	if( (txt_hpos >= 20) && (txt_hpos < 20 + 10) )/*digits 0 trough 9*/
		{
		*selected_item = txt_hpos - 20;/* 0 to 9 is digits */
		return(1);
		}
	if(txt_hpos == 30)/*'C' clear entry*/
		{
		*selected_item = CLEAR_ENTRY;
		return(1);
		}
	if(txt_hpos == 31)/* '?' reveal / conceal toggle, default conceal */
		{
		*selected_item = REVEAL_CONCEAL_TOGGLE;
		return(1);
		}
	if(txt_hpos == 32)/*'R' restore previous page (from RAM)*/
		{
		*selected_item = RESTORE_PREVIOUS_PAGE;
		return(1);
		}
	if(txt_hpos == 33)/*save page*/
		{
		*selected_item = SAVE_PAGE;
		return(1);
		}
	if(txt_hpos == 34)/*'L' load page*/
		{
		*selected_item = LOAD_PAGE;
		return(1);
		}
	if(txt_hpos == 35)/* 'E' erase page */
		{
		*selected_item = ERASE_PAGE;
		return(1);
		}
	if(txt_hpos == 36)/*'P' program vcr with times*/
		{
		*selected_item = PROGRAM_VCR;
		return(1);
		}
	if(txt_hpos == 37)/*'B' bank 0 / 1 toggle*/
		{
		*selected_item = ACQUISITION_BANK_TOGGLE;
		return(1);
		}
	if(txt_hpos == 38)/* 'D' delete last timer */
		{
		*selected_item = DELETE_LAST_TIMER;
		return(1);
		}
	if(txt_hpos == 39)/* 'G' group 1 / 2 toggle */
		{
		*selected_item = DISPLAY_GROUP_TOGGLE;
		return(1);
		}
	}/*end special field processing*/
if( (*selected_display_page == 1) && (txt_vpos == 0) ) /*on macro clickbar */
	{
	if(txt_hpos == 31) /* macro 'A' */
		{
		*selected_item = MACRO_A;
		return(1);
		}
	if(txt_hpos == 32) /* macro 'B'*/
		{
		*selected_item = MACRO_B;
		return(1);
		}
	if(txt_hpos == 33) /* macro 'C' */
		{
		*selected_item = MACRO_C;
		return(1);
		}
	if(txt_hpos == 34) /* macro 'D' */
		{
		*selected_item = MACRO_D;
		return(1);
		}	
	if(txt_hpos == 35) /* macro 'E' */
		{
		*selected_item = MACRO_E;
		return(1);
		}
	if(txt_hpos == 36) /* macro 'F'*/
		{
		*selected_item = MACRO_F;
		return(1);
		}
	if(txt_hpos == 37) /* macro 'G' */
		{
		*selected_item = MACRO_G;
		return(1);
		}
	if(txt_hpos == 38) /* macro 'H' */
		{
		*selected_item = MACRO_H;
		return(1);
		}	
	if(txt_hpos == 39) /* macro 'I' */
		{
		*selected_item = MACRO_I;
		return(1);
		}
	}/* end macro click bar */
/*get the page*/
saa_dsp_mem = *selected_display_page + dspmem_offsett;
if(!get_integer_value_pointed_at(saa_dsp_mem, txt_hpos,txt_vpos,
										selected_item, selected_item_tail))
	{
	return(0);/*hardware error, selected_item is -1 for invalid data*/
	}
/*check for valid result*/
if(*selected_item == REVEAL_CONCEAL_TOGGLE)/* on '?' */
	{
	return(1);
	}
if( (*selected_item < 100) || ( *selected_item > 899) )
										/*not a txt page value pointed at*/
	{
	*selected_item = -1;
	return(1);
	}
/* *selected_item and selected_display_page now -1 for error*/
/*sprintf(temp, "hpos = %d  vpos = %d  display = %d, selected_item = %d",\
/*txt_hpos, txt_vpos, *selected_display_page, *selected_item);
/*s3putmsg(0, 400, 14, 12, temp, 1);
*/
return(1);
}

print_in_display(int selected_display_page, char *text)
{
int fg, bg;
int x, y;
fg = YELLOW;
bg = RED;
if(selected_display_page == 0)
	{
	x = 0;
	y = 225;/*25 * 9*/
	}
if(selected_display_page == 1)
	{
	x = 320;
	y = 225;
	}
if(selected_display_page == 2)
	{
	x = 0;
	y = 465;/*25 * 9 + (480 / 2)*/
	}
if(selected_display_page == 3)
	{
	x = 320;
	y = 465;
	}
/* draw the bar to click on*/
/* 0 trough 9 stand for digits making up page,*/
/* 0 at position 20 stands for digit 0 for entering pages and subpages,
/* 1 at position 21 stands for digit 1 for entering pages and subpages,
/* etc., through 9 at position 29.
/* C at position 30 stands for clear (page and subpage entry),
/* ? at position 31 stands for reveal / conceal toggle */
/* R at position 32 stands for restore (get previous page in this display),
/* S at position 33 stands for save page (filename asked for),
/* L at position 34 stands for load page (filename asked for),
/* E at position 35 stands for clear screen,
/* P at position 36 stands for program vcr with timers,
/* B at position 37 stands for bank,
/* D at position 38 stands for delete last timer programmed,
/* G at position 39 stands for display group 1 / _2 toggle.
*/
/* prevent text getting mixed up */
if(strlen(text) < 20)/* print click points in second part of line */
	{
	s3putmsg(x, y, fg, bg, "                    0123456789C?RSLEPBDG", 1);
	}
else /* clear whole line */
	{
	s3putmsg(x, y, fg, bg, "                                        ", 1);
	}
s3putmsg(x, y, fg, bg, text, 1);/* print meesage */
return(1);
}	

xacq_page(int selected_display_page)/* acquire the corresponding page */
/* if bank 0 selected, and dspmem_offsett is 4, not visible,
/* if bank 1 selected, and dspmem_offset is 0, not visible.
*/
{
int bank;
get_saa_bank(&bank);
if(! acqpag(/*page, time, bank, acqcir, timed mode, hold*/\
_main_page[ selected_display_page ],\
_sub_page[ selected_display_page ],\
bank,\
_acquisition_circuit[ selected_display_page ],\
_timed_page_flag[ selected_display_page ],\
_hold_page_flag[ selected_display_page ]) )
	{
	return(0);/*hardware error*/
	}
return(1);
}/*end function xacq_page*/

hold_page(int selected_display_page, int dspmem_offsett)
/*hold acquisition for this page*/
{
int bank;
get_saa_bank(&bank);
/* check if visible, if not, no need to hold */
/* do not hold if active in other display */
if( (last_active_bank == 0) && (dspmem_offsett == 4) )return(1);
if( (last_active_bank == 1) && (dspmem_offsett == 0) )return(1);/*not visible*/
/*if( (bank == 0) && (dspmem_offsett == 4) )return(1);/*not visible*/
/*if( (bank == 1) && (dspmem_offsett == 0) )return(1);/*not visible*/
/* looking at live acq */
/* set the acquisition circuit*/
_acquisition_circuit[selected_display_page] = selected_display_page;
/*page, time, bank, acqcir, timed mode, hold*/\
if(! acqpag(
_main_page[ selected_display_page ],\
_sub_page[ selected_display_page ],\
bank,\
_acquisition_circuit[ selected_display_page ],\
_timed_page_flag[ selected_display_page ],\
_hold_page_flag[ selected_display_page ]) )
	 {
	 return(0);/*hardware error*/
	 }
return(1);
}/*end function hold_page*/

mouse_handling(int *dspmem_offsett)
{
/*following saved in between calls to this function*/
static int mode;

static int selected_display_page;/*page where the mouse click happens, also
									/*pointer in static global arrays*/
static int selected_page;/*the mouse was pressed on this 3 digit page nmbr*/
static int selected_subpage;
static int selected_item;
static int selected_item_tail;
static int accumulator;
static int accumulation_cnt;

static char usertext[80];
int m_flags;
char temp[80];
char path[40];
int fgcolor;
int dummy;
int x, y;
int a, b, c, d, e;
int i;
int key;
static int bank;
static int timeptr;
static int total;
extern int audio_volume;
FILE *exec_filefd;

/*start up initialisation, saa chip may still be active from previous execution
/*of this program*/
if(! x_initiated_flag)/* cold start */
	{
	/*copy the present status of the saa chip to the memory array*/
	bank = 0;
	for(i = 0; i < 4; i++)/*for all used display memories in saa RAM*/
		{
		/* get present subpage and page from saa RAM */
		if(! getsupa(i, &selected_subpage, &selected_page) )return(0);
															/*hardware error*/
		_main_page[i] = selected_page;
		_sub_page[i] = selected_subpage;
		_acquisition_circuit[selected_display_page] = i;
		if(selected_subpage == 0)
			{
			_timed_page_flag[i] = 0;	/*not a timed page*/
			}
		else _timed_page_flag[i] = 1;
		_hold_page_flag[i] = 0;		/*no hold*/
		/* create a bar (to click on) in the selected group (x or y)*/
		/* NOTE: Unfortunatly it does not seem to be possible to read the hold
		/* status of each acquisition circuit from the saa chip */
		sprintf(temp, "%03d/%03d UNDEFINED", _main_page[i], _sub_page[i]);
		print_in_display(i + *dspmem_offsett, temp);
		/* print the macro clickbar */
		}/*end for all used display memories in saa chip*/
	macro_flag = MACRO_OFF;/* not in macro on cold start */
	print_macro_status(MACRO_A, MACRO_OFF);/* macro a used as dummy */
	accumulator = 0;
	accumulation_cnt = 0;
/*	*dspmem_offsett = 0;/* start with saa display memory 0 trough 3 */
	mode = GET_PAGE;
	selected_display_page = 0;
	x_initiated_flag = 1;
	}/* end if x_initiated_flag = 0, (cold start) */
/* check if x mode was temporarely left, but not program */
if(x_initiated_flag == 2)   /* left x mode, now back, set by main case 'x' on
							/* exit of call to displaytxtpages, also
							/* if F9 after display txt char set
							/* and when right mouse click on clickbar 'E' */
	{
	/* print the click bar, and the status of each acquisition circuit,
	/* in each display page */
	print_status();
	macro_flag = MACRO_OFF;
	print_macro_status(MACRO_A, MACRO_OFF);/* macro a used as dummy */
	x_initiated_flag = 3;/* prevent doing previous every time */
	}

while(1)/* events cueued or macro read */
	{
	if(macro_flag == MACRO_READ)	/* reading from macro file */
		{
		if( fscanf(macrofile, "%d %d %d %d %d", &a, &b, &c, &d, &e) == EOF)
			{
			fclose(macrofile);
			print_macro_status(MACRO_A, MACRO_OFF);/* macro a used as dummy */
			macro_flag = MACRO_OFF;
			return(1);
			}
		else	/*reading from macro file */
			{
			myevent.type = a;
			myevent.xbutton.button = b;
			myevent.xbutton.x = c;
			myevent.xbutton.y = d;
			text[0]= e;
			}
		/* else continue with these values */
		}/* end if macro flag == MACRO_READ */
	else /* read form user */
		{
		a = XEventsQueued(mydisplay, QueuedAfterReading);
		if(!a)break;	
		XNextEvent(mydisplay, &myevent);
		}	
	switch (myevent.type)
		{
		case MappingNotify:
			XRefreshKeyboardMapping((XMappingEvent *) &myevent);
			break;
		case ConfigureNotify:
			recalculate(myevent.xconfigure.width,\
				myevent.xconfigure.height);
			XMapSubwindows(mydisplay, topwindow);
			break;
		case ButtonPress:
			if(macro_flag == MACRO_WRITE)
				{
				if(macrofile)/* writing to macro file */
					{
 					/* only store left button,
					/* else error messages for click on macro bar in
					/* macro read*/
					if(myevent.xbutton.button == M_LEFT_DOWN)
						{
						fprintf(macrofile, "%d %d %d %d %d ",\
							myevent.type,\
							myevent.xbutton.button,\
							myevent.xbutton.x,\
							myevent.xbutton.y,\
							text[0]);
						}	
	 				}
 				}/* end macro write */
			mouse_click = 1;
			print_cursor_status(\
			myevent.xbutton.x, myevent.xbutton.y, *dspmem_offsett);

			if(myevent.xbutton.button == M_LEFT_DOWN)
			/*calculate txt page from value pointing at*/
				{
				if( (mode == GET_PAGE) || (mode == GET_SUBPAGE) )
					{
					/*get what the mouse is pointing at*/
					calculate_teletext_page_and_position(\
						myevent.xbutton.x, myevent.xbutton.y,\
						&selected_item, &selected_item_tail,\
						&selected_display_page, *dspmem_offsett);
					/* clear any previous messages */
					print_status();
					/*range check*/
					if( (selected_item >= MACRO_A) && (selected_item <= MACRO_I) )
					/*macro read*/
					/* a click in macro bar */
						{
						if(macro_flag == MACRO_OFF)/* valid */
						    {
							temp[0] = selected_item - MACRO_A + 'A';
							temp[1] = 0;/* string termination */
							get_path_name("macro_files_dir", path);
							sprintf(present_macro_name,\
								 "%s%s.mac", path, temp);
							macrofile = fopen(present_macro_name, "r");
														/* a.mac, b.mac etc. */
							if(macrofile)
								{
								sprintf(temp, "READING FROM MACROFILE %s",\
								present_macro_name);
								print_in_display(selected_display_page, temp);
								print_macro_status(selected_item, MACRO_READ);
								macro_flag = MACRO_READ;
								return(1);
								}
							print_in_display(selected_display_page, "NO MACRO DEFINED");
							return(1);
							}/* end if valid */
						}/* end left click user on macro (macro read command) */
					if( (selected_item > 99) && (selected_item < 900) )/*valid*/
						{
						if(mode == GET_PAGE)
							{
							selected_page = selected_item;
							if(selected_item_tail != -1)/*130/1 or like that*/
								{
								selected_subpage = selected_item_tail;
								}
							else selected_subpage = 0;
							}
						}/*end valid page or page with subpage extention*/
					if(selected_item == SAVE_PAGE)
						{
						print_in_display(selected_display_page, "SAVE name?");
						return(1);/* key event handles rest */
 						}	
					if(selected_item == LOAD_PAGE)
						{
						print_in_display(selected_display_page, "LOAD name?");
						/* save this display page in RAM for restore function*/
						if(!put_display_page_in_ram(selected_display_page,\
						*dspmem_offsett))
							{
							return(0);/*hardware error*/
							}
						/*set hold*/
						_hold_page_flag[selected_display_page] = 1;
						/*send hold*/
						if(! hold_page(selected_display_page, *dspmem_offsett) )
							{
							return(0);
							}
						return(1);/* keyevent handles rest */
						}
					if(selected_item == ERASE_PAGE)
						{
						if(! tcls(selected_display_page + *dspmem_offsett) )return(0);
						print_in_display(selected_display_page, "page erased");
						return(1);
						}
					if(selected_item == DISPLAY_GROUP_TOGGLE)
							 					/* default = 0 */
						{ 
						*dspmem_offsett = 4 - *dspmem_offsett;
						if(*dspmem_offsett == 0)/* saa memory 0 trough 3 */
							{
							print_in_display(selected_display_page, "group X selected");
							}
						else	/* saa memory 4 trough 7 */
							{
							print_in_display(selected_display_page, "group Y selected");
							}
						return(1);
						}
					if(selected_item == ACQUISITION_BANK_TOGGLE)
						{
						bank = 1 - bank;/*bank toggle*/
						sprintf(temp, "bank %1d selected", bank);
						print_in_display(selected_display_page, temp);
						set_saa_bank(bank);
						return(1);
						}
					if(selected_item == PROGRAM_VCR)
						{	
						/* clear any previous messages */
						print_status();
						get_time_ptr(&timeptr);
						if(!timeptr)
							{
							print_in_display(selected_display_page, "No timers set");
							return(1);
							}
						print_in_display(selected_display_page,\
						"Programming vcr, one moment             ");
						if(!vcr_prog())
							{
							print_in_display(selected_display_page,\
							"Cannot program vcr");
							return(1);
							}
						print_in_display(selected_display_page,\
						"Ready programming vcr, please verify!  ");
						return(1);
						} /* end programming vcr */
					if(selected_item == DELETE_LAST_TIMER)
						{
						get_time_ptr(&timeptr);
						if(timeptr > 0)
							{
							timeptr--;
							set_time_ptr(timeptr);
							gtotal(&total);
							sprintf(temp, "Last timer erased, total now %d minutes",\
							total);
							print_in_display(selected_display_page, temp);
							}
						return(1);
						}
					if(selected_item == RESTORE_PREVIOUS_PAGE)
						{
						if(! get_display_page_from_ram(selected_display_page,\
						*dspmem_offsett, _hold_page_flag) )
							{
							return(0);/*hardware error*/
							}	
						return(1);
						}
					if(selected_item == CLEAR_ENTRY)/*clear entry*/
						{
						mode = GET_PAGE;
						selected_page = 0;
						selected_subpage = 0;
						accumulator = 0;
						accumulation_cnt = 0;
						print_in_display(selected_display_page, "CLEAR");
						delay(50);
						print_status();
						return(1);
						}
					if(selected_item == REVEAL_CONCEAL_TOGGLE)
										/* default conceal*/
						{
						/* toggle */
						_reveal_flag[selected_display_page + *dspmem_offsett] =\
						1 - _reveal_flag[selected_display_page + *dspmem_offsett];
						if(_reveal_flag[selected_display_page + *dspmem_offsett] == 1)
							{
							print_in_display(selected_display_page, "REVEAL");
							}
						if(_reveal_flag[selected_display_page + *dspmem_offsett] == 0)
							{
							print_in_display(selected_display_page, "CONCEAL");
							}
						return(1);
						}
					if( (selected_item >= 0) && (selected_item <= 9) )/*single digit*/
						{
						accumulator = accumulator * 10 + selected_item;
						/*feedback to user*/
						if(mode == GET_PAGE)
							{
							sprintf(temp, "%03d/%03d PAGE?",\
							accumulator, selected_subpage);
							}
					
	
	
	
						if(mode == GET_SUBPAGE)
							{
							selected_subpage = accumulator;
							sprintf(temp, "%03d/%03d SUBPAGE?",
							selected_page, selected_subpage);
							}
						print_in_display(selected_display_page, temp);
						accumulation_cnt++;
						if(accumulation_cnt == 3)/*3 digits could be a page*/
							{	
							/*overrule previous*/
							if(mode == GET_PAGE)
								{
								selected_page = accumulator;
								selected_subpage = 0;
								}
							accumulator = 0;
							accumulation_cnt = 0;	
							}/*end full field*/
						else /*proces next digit*/
							{
							if(accumulator > 899)/*not valid txt page or subpage*/
								{
								print_in_display(selected_display_page, "NOT VALID");
								accumulator = 0;
								accumulation_cnt = 0;
								return(1);/*end digits, invalid selection*/
								}/*end overflow*/
							continue;/*get next digit or command*/
							}/*end proces next digit*/
						}/*end single digit*/
					else/*not a single digit*/
						{
						/*test if early end of subpage*/
						if(mode == GET_SUBPAGE)
							{
							accumulator = 0;
							accumulation_cnt = 0;
							}
						}/*end not a single digit*/
		
					if(selected_item == -1)/*invalid selection*/
						{
						/*if click not on status bar in get subpage mode,
						/*the subpage digit field is considered complete,*/
						/*and an acquisition is performed*/
						if(mode == GET_SUBPAGE)
							{
							/* clear info bar in this page */
							print_in_display(selected_display_page, " ");
							goto acquire;
							}
						print_in_display(selected_display_page, "NOT A PAGE");
						continue;/*do nothing*/
						}
					sprintf(temp, "%03d/%03d SUB/DEST?",\
					selected_page, selected_subpage);
					print_in_display(selected_display_page, temp);
					mode = GET_SUBPAGE;
					continue;/*go for next mouse click*/
					}/*end mode is GET_PAGE or GET_SUBPAGE*/
				if(mode == SELECTED)	/*can get it now*/
					{
					/*calculate the new selected_display_page from the mouse position*/
					calculate_teletext_page_and_position(\
						myevent.xbutton.x, myevent.xbutton.y,\
						&selected_item, &dummy,\
						&selected_display_page, *dspmem_offsett);
					/*arguments are x, y, &page, &page display, offsett*/
acquire:
					/* first save this display page in RAM for restore function */
					/* save this display in ram for memory functions (F7 to F10) */
					if(! save_page_in_ram_file(selected_display_page,\
					_main_page[selected_display_page],\
					_sub_page[selected_display_page]) )
						{
						return(0);/* hardware error */
						}
					/* save page in ram file for 'r' function (recall) */
					if(!put_display_page_in_ram(selected_display_page, *dspmem_offsett))
						{
						return(0);/*hardware error*/
						}
					/*get page in acqcir display_page*/
					/*acquire a txt page, use selected_page from first mouse click,
					/*and selected_display_page from second mouse click*/
					_main_page[ selected_display_page ] = selected_page;
					_sub_page[ selected_display_page ] = selected_subpage;
					bank,\
					_acquisition_circuit[ selected_display_page ] =\
					selected_display_page;
					if(_sub_page[ selected_display_page ] == 0)
						{
					/*select all subpages*/
					_timed_page_flag[ selected_display_page ] = 0;
						}
					else
						{
						_timed_page_flag[ selected_display_page ] = 1;
						}
					_hold_page_flag[ selected_display_page ] = 0;/*no hold*/
					if(! xacq_page(selected_display_page) )return(0);/*hardware error*/
					selected_subpage = 0;/*reset*/
					/* print status bars in all display pages */
					print_status();
					mode = GET_PAGE;
					return(1);
					}
				}/*end if left mouse button pressed*/
			if(myevent.xbutton.button == M_MIDDLE_DOWN)/*hold toggle page in selected page*/
				{
				/*find out where we are*/
				calculate_teletext_page_and_position(\
					myevent.xbutton.x, myevent.xbutton.y,\
					&dummy, &dummy,\
					&selected_display_page, *dspmem_offsett);
				/*togle hold for the corresponding acquisition circuit*/
				_hold_page_flag[ selected_display_page ] =\
				1 - _hold_page_flag[ selected_display_page ];
				/*report in destination screen*/
				print_status();
				/*send the hold*/
				if(! hold_page(selected_display_page, *dspmem_offsett) )return(0);
				return(1);
				}/* end if middle mouse button pressed */
			if(myevent.xbutton.button == M_RIGHT_DOWN)
				/* program vcr with times found at cursor, create macro, 
								/* or end 'X' mode */
				{								
				/* clear any previous messages */
				print_status();
				/* find out where we are */
				calculate_teletext_page_and_position(\
					myevent.xbutton.x, myevent.xbutton.y,\
					&selected_item, &dummy,\
					&selected_display_page, *dspmem_offsett);
				if( (selected_item >= MACRO_A) && (selected_item <= MACRO_I) )
				/* a click in macro bar */
					{
					if(macro_flag == MACRO_WRITE)/* close macro file */
						{
						if(macrofile)fclose(macrofile);
						sprintf(temp, "MACRO FILE %s CLOSED", present_macro_name);
						print_in_display(selected_display_page, temp);
						print_macro_status(selected_item, MACRO_OFF);
						macro_flag = MACRO_OFF;
						return(1);
						}
					if(macro_flag == MACRO_OFF)/* open macro file for write */
					    {
						temp[0] = selected_item - MACRO_A + 'A';
						temp[1] = 0;/* string termination */
						get_path_name("macro_files_dir", path);
						sprintf(present_macro_name,\
							 "%s%s.mac", path, temp);
						macrofile = fopen(present_macro_name, "w");
												/* a.mac, b.mac etc. */
						if(macrofile)
							{
							sprintf(temp, "MACRO FILE %s OPEN FOR WRITE",\
							present_macro_name);
							print_in_display(selected_display_page, temp);
							print_macro_status(selected_item, MACRO_WRITE);
							macro_flag = MACRO_WRITE;
							return(1);
							}
						sprintf(temp, "CANNOT OPEN MACRO FILE %s FOR WRITE",\
						present_macro_name);
						print_in_display(selected_display_page, temp);
						return(1);
						}
					}/* end click in macro bar */
				if(selected_item == ERASE_PAGE)
					{
					x_initiated_flag = 2;/* indicate xmode temp left */
					return(0);/* leave mode 'X' */
					}
				if(myevent.xbutton.y < 240)y = myevent.xbutton.y / charheight;
				else y = (myevent.xbutton.y - 240) / charheight;
/*				sprintf(temp,\
/*				"WAS HERE myevent.xbutton.y=%d y=%d sel dsp=%d off=%d",\
/*				myevent.xbutton.y, y, selected_display_page, *dspmem_offsett);
/*				s3putmsg(0, 10, WHITE, BLUE, temp, 1);
*/
				xfndtime(selected_display_page + *dspmem_offsett,\
				y, ".", selected_display_page);
				}/* end if right mouse button pressed */
			break;

		case KeyPress:
			if(macro_flag == MACRO_READ)
				{
				;/* do nothing, everything in already */
				}
			else
				{
				if (XLookupString((XKeyEvent *) &myevent,\
					text, 10, &mykey, 0) != 1)break;
				}
			if(macro_flag == MACRO_WRITE)
				{
				if(macrofile)/* writing to macro file */
					{
 					fprintf(macrofile, "%d %d %d %d %d ",\
						myevent.type,\
						myevent.xbutton.button,\
						myevent.xbutton.x,\
						myevent.xbutton.y,\
						text[0]);
	 				}
 				}/* end macro write */
			if(selected_item == SAVE_PAGE)
				{
				/*x, y, max length, selected display, string*/
				/*25 for on info bar*/
				if(xget_input_string_with_echo(11, 25, 8,\
					selected_display_page, usertext, text[0]) == EOF)
					/* enter was pressed */
					{
					if(! save_page(usertext, selected_display_page,\
						 *dspmem_offsett) )
						{
						return(0);/*hardware error*/
						}/* end user pressed enter */
					}/* end enter was pressed */
				}/* end selected_item == SAVE_PAGE */
			else if(selected_item == LOAD_PAGE)
				{
				/*x, y, max length, selected display, string*/
				/*25 for on info bar*/
				if(xget_input_string_with_echo(11, 25, 8,\
				selected_display_page, usertext, text[0]) == EOF)
					/* enter was pressed */
					{
					/*set hold*/
					_hold_page_flag[selected_display_page] = 1;
					/*send hold*/
					if(! hold_page(selected_display_page,\
						 *dspmem_offsett) )
						{
						return(0);
						}
					/*now load the page from disk*/
					if(! load_page(usertext, selected_display_page,\
						 *dspmem_offsett) )
						{
						return(0);/*hardware error*/
						}
					}/* end user pressed enter */
				}/*end selected_item == LOAD_PAGE */
			else switch(toupper(text[0]))
				{
				case ESC:/* exit xkrs disabled, dangerous if in tv mode,
							/* no way to switch back */
				/* exit(0);*/
					break;
				}/* end switch toupper text[0] */
			default:
				break;
			}/* end switch myevent type */
		/* handle special keys here (using key code) */	
		switch(myevent.xbutton.button)
				{		
				case GREY_PAGE_UP:	/* saa dspmem and bank change*/
				case GREY_PAGE_DOWN:
				case GREY_HOME:
					display_select(text[0]);
					break;
				case F1:	/* select teletext via panteltje txt decoder */
					video_control(TELETEXT);
					break;
				case F2:	/* select tv program,
							/* using vga video philips dtv1000 tv card */
					video_control(TV_PROGRAM);
					break;
				case F3:	/* teletext via pal coder output */
					video_control(AUX);		
					break;				
				case F5:/* volume down */
					if(audio_volume > 0) audio_volume--;
					sprintf(temp, "cmix vol %d", audio_volume);
					exec_filefd = popen(temp, "r");
					pclose(exec_filefd);
					if(! exec_filefd) break;
					break;
				case F6:/* volume up */
					if(audio_volume < 100) audio_volume++;
					sprintf(temp, "cmix vol %d", audio_volume);
					exec_filefd = popen(temp, "r");
					if(! exec_filefd) break;
					pclose(exec_filefd);
					break;					
				case F7:/* display pages in ramfile home (first entry) */
					_hold_page_flag[0] = 1;			/* set hold flag */
					hold_page(0, *dspmem_offsett);	/* send the hold */
					load_page_from_ram_file(FIRST, 0, *dspmem_offsett);
					break;
				case F8:/* display pages in ramfile up */
					_hold_page_flag[0] = 1;			/* set hold flag */
					hold_page(0, *dspmem_offsett);	/* send the hold */
					load_page_from_ram_file(PREVIOUS, 0, *dspmem_offsett);
					break;
				case F9:/* display pages in ramfile down */
					_hold_page_flag[0] = 1;			/* set hold flag */
					hold_page(0, *dspmem_offsett);	/* send the hold */
					load_page_from_ram_file(NEXT, 0, *dspmem_offsett);
					break;
				case F10:/* display pages in ramfile end (last entry) */
					_hold_page_flag[0] = 1;			/* set hold flag */
					hold_page(0, *dspmem_offsett);	/* send the hold */
					load_page_from_ram_file(LAST, 0, *dspmem_offsett);
					break;
				
				}/* end switch myevent.xbuttton.button */

/*
	sprintf(temp, "x=%d y=%d b=%d k=%d",
	myevent.xbutton.x, myevent.xbutton.y, myevent.xbutton.button,\
	text[0]);
	print_in_display(0, temp);
	printf("\n%s\n", temp);
*/
	}/* end while events queued */
return(1);
}/* end function mouse handling */

displaytxtpage()
/* display teletext pages in dspmem 0 trough 3 (if dspmem_offsett is 0) or
/* 4 trough 7 (if dspmem_offsett is 4)*/
{

static int dspmem_offsett;
static int flash;/* 0 if character on, 1 if off,
					/*  toggles on call to this function */

int a, b, c, d;
int x, y, p;
int i;
int ptr, ptrb, ptrp;
char cctram_data[1028];
char temp[2];

int fgcolor;
int bgcolor;
int graphicsflag;
int gholdflag;		/*hold graphics*/
int gsepaflag;		/*separated graphics*/
int concealflag;
int boxflag;
int soflag;			/*SO SI unknown command*/
int escapeflag;
int flashflag;
int dheightflag;	/*double height*/
int dheightline;
int posx[4];
int posy[4];
/* remember where in this line was double height,
/* need to clear the x positions that were not double height in this line,
/* one line lower, else old data will remain visible there. */
char dheight_xpositions_background[TXT_COLUMNS];
int last_graphic_character;
int last_background;
int last_foreground;
int foreground;
int background;
int key;

/* select character set, global statics, used by many functions */
charheight = S14_CHAR_HEIGHT;
charwidth = S14_CHAR_WIDTH;

/*if(fy != stdin)return(0);/*reading from file*/

posx[0]=0;			  		/*top left*/
posy[0]=0;

posx[1]=(HSIZE / 2) + 1;	/*top middle, avoid overwriting window line*/
posy[1]=0;

posx[2]=0;					/*middle left, avoid overwriting window line*/
posy[2]=(VSIZE / 2) + 1;

posx[3]=(HSIZE / 2) + 1;	/*middle middle, avoid overwriting window line*/
posy[3]=(VSIZE / 2) + 1;


/*draw 4 windows*/
/*vertical in middle top to bottom*/
/* s3unpaint_mouse must know about pixels,
/* only s3plot stores them in array (see xglgraphi.c)*/
for(y = 0; y < VSIZE; y++)
	{
	s3plot(HSIZE / 2, y, WHITE);
	}
/*horizontal in middle left to right*/
for(x = 0; x < HSIZE; x++)
	{
	s3plot(x, VSIZE / 2, WHITE);
	}

flash = 1 - flash; /* toggle */
for(p = 0; p < 4; p++)/*display 4 dspmems*/
	{
	if(! pgetstr(0, 0, p + dspmem_offsett, PAGLEN, cctram_data) )return(0);
								/*error reading teletext data*/
	/* the following flags and colors are reset at the start of every page */
	/* none */
	for(y = 0; y < TXT_ROWS; y++)/*every line in txt page*/
		{
		if(! mouse_handling(&dspmem_offsett) )
			{
			return(0);
			/* exit(2); */
			}	
		/* mouse, x or y mode in mouse routine changes dspmem_offsett from 0
		/* for x mode to 4 for y mode,
		/* dspmem_offsett is initialized to 0 on first call to mouse_handling*/
		
		/*the following flags are reset at the start of each line*/
		graphicsflag = 0; /* not if graphics hold */
		gsepaflag = 0;
		bgcolor = BLACK;
		fgcolor = WHITE;
		escapeflag = 0;
		flashflag = 0;
		dheightflag = 0;
		concealflag = 0;
		dheightflag = 0;
		gholdflag = 0;
		boxflag = 0;
		soflag = 0;
		dheightline = 0;
		ptr = y * TXT_COLUMNS;

		for(x = 0; x < TXT_COLUMNS; x++)/*every column in txt page*/
			{
			ptrb = ptr + x;
			if(ptrb > PAGLEN)break;
			a = cctram_data[ptrb];			/*get data*/
			if(a > 31)		/*not a control character*/
				{
				/* 0x20 trouhg 0x3f first 32 graphics characters teletext */
				/* 0x60 trough 0x7f second 32 */
				/* s14font graphics 0xa0 through 0xff (is 160 trough 255)
				/* first half s14font graphics at 0xa0,
				/* second half s14font graphics at 0xe0.
				*/
				if(graphicsflag)	/*display as graphics if valid, else text */
					{
					if( (a >= 0x20) && (a <= 0x3f) )	/* upper group */
						{
						a += (0xa0 - 0x20);/* upper half of graphics set s14font*/
						}
					if( (a >= 0x60) && (a <= 0x7f) ) /* lower group */
						{
						a += (0xe0 - 0x60);/* lower half of graphics set s14font */
						}
					} /* end graphics flag */
				/* print code here */
				temp[0] = a;
				temp[1] = 0;/* string termination */
				/* only print clock in display 0 display_offset 0 */
				/*date and time only correct in selected display memory,
				/*date starts here, followed by time,
				/*print spaces to end of line*/
				get_saa_dspmem(&a);
				if( ((p + dspmem_offsett) != a) && (y == 0) && (x >= 21) )
					{
					temp[0] = ' ';/* print space */
					}
				/* reveal handling */
				if( (! _reveal_flag[p + dspmem_offsett])  &&	concealflag )
				/* not reveal request by user, and conceal by transmittor */
						{
						temp[0] = ' ';/* print space */
						}
				/* flash handling */
				/* print char if flash on, else overwrite with space */
				if( flashflag && (!flash) )/* flash was off */
					{
					temp[0] = ' ';/* print space */
					}
				/* do not overwrite mouse position display and macro clickbar
				/* in display 1 (top right) */
				if( (p == 1) && ( y == 0) && (x >= 21) ){}/* do nothing */
				else
					{
					if(dheightflag)
						{
						/* remember where and what bgcolor */
						dheight_xpositions_background[x] = -1;
						s3txtdoubleputmsg(\
						posx[p] + (x * charwidth),\
						posy[p] + (y * charheight),\
						fgcolor, bgcolor, temp, gsepaflag);
						}
					else
						{
						dheight_xpositions_background[x] = bgcolor;
						s3txtputmsg(\
						posx[p] + (x * charwidth),\
						posy[p] + (y * charheight),\
						fgcolor, bgcolor, temp, gsepaflag);
						}
					}
				last_graphic_character = temp[0];/* save for graphics hold */
				last_background = bgcolor;
				last_foreground = fgcolor;
				}/* end not a control character */
			else		/* < 32 is control characters*/
				{
				concealflag = 0;/* reset conceal flag*/
				switch(a)				/*control character sort*/
					{
					case 0:
						fgcolor = BLACK;
						graphicsflag = 0;
						break;
					case 1:
						fgcolor = RED;	/* using intensified, 
										/* luminance of red to low to read*/
						graphicsflag = 0;
						break;
					case 2:
						fgcolor = GREEN;
						graphicsflag = 0;
						break;
					case 3:
						fgcolor = YELLOW;
						graphicsflag = 0;
						break;
					case 4:
						fgcolor = BLUE;
						graphicsflag = 0;
						break;
					case 5:
						fgcolor = MAGENTA;
						graphicsflag = 0;
						break;
					case 6:
						fgcolor = CYAN;
						graphicsflag = 0;
						break;
					case 7:
						fgcolor = WHITE;
						graphicsflag = 0;
						break;
					case 8:				/*flash*/
						flashflag = 1;
						break;
					case 9:				/*steady*/
						flashflag = 0;
						break;
					case 10:			/*end box*/
						boxflag = 0;
						break;
					case 11:			/*start box*/
						boxflag = 1;
						break;
					case 12:			/*normal height*/
						dheightflag = 0;
						break;
					case 13:			/*double height*/
						dheightflag = 1;
						dheightline = 1;/* flag if set causes next line to be 
										/* erased at x pos. where no dheight in
										/* this line */
						break;
					case 14:			/*SO unknown command*/
						soflag = 1;
						break;
					case 15:			/*SI unknown command*/
						soflag = 0;
						break;
					case 16:
						fgcolor = BLACK;
						break;
					case 17:
						fgcolor = RED;	/* using intensified,
										/* luminance od red to low to read */
						graphicsflag = 1;
						break;
					case 18:
						fgcolor = GREEN;
						graphicsflag = 1;
						break;
					case 19:
						fgcolor = YELLOW;
						graphicsflag = 1;
						break;
					case 20:
						fgcolor = BLUE;
						graphicsflag = 1;
						break;
					case 21:
						fgcolor = MAGENTA;
						graphicsflag = 1;
						break;
					case 22:
						fgcolor = CYAN;
						graphicsflag = 1;
						break;
					case 23:
						fgcolor = WHITE;
						graphicsflag = 1;
						break;
					case 24:
					 	concealflag = 1;
						break;
					case 25:			/*continuous graphics*/
						gsepaflag = 0;
						break;
					case 26:			/*separated graphics*/
						gsepaflag = 1;
						graphicsflag = 1;
						break;
					case 27:			/*escape*/
						escapeflag = 1;
						break;
					case 28:			/*black background*/
						bgcolor = BLACK;
						break;
					case 29:			/*new background*/
						gholdflag = 0;
						bgcolor = fgcolor;
						break;
					case 30:			/*hold graphics*/
						gholdflag = 1;
/*						graphicsflag = 1;*/
						break;
					case 31:			/*release graphics*/
						gholdflag = 0;
						break;
						}
				/*control characters appear as spaces*/
				if(gholdflag && graphicsflag)/* repeat last graphics char */
					{
					foreground = last_foreground;
					background = last_background;
					temp[0] = last_graphic_character;
					}
				else
					{
					foreground = fgcolor;
					background = bgcolor;
					temp[0] = ' ';/* space */
					}
				temp[1] = 0;/* string termination */
				/* leave macro clickbar in display 1 (top right) */
				if( (p == 1) && (y == 0) && (x >= 30) ){}/* do nothing */
				else
					{
					if(dheightflag)
						{
						dheight_xpositions_background[x] = -1;/* remember where */
						s3txtdoubleputmsg(\
						posx[p] + (x * charwidth),\
						posy[p] + (y * charheight),\
						foreground, background, temp, gsepaflag);
						}
					else
						{
						dheight_xpositions_background[x] = background;
						s3txtputmsg(\
						posx[p] + (x * charwidth),\
						posy[p] + (y * charheight),\
						foreground, background, temp, gsepaflag);
						}
					}
				}/*end control characters*/
			}/*end for each column*/
		/* dheightline should remain to end of line */
		/* only prevents line being overwritten */
		/* dheightflag may be reset within line */
		if(dheightline)
			{
			/* Has now printed double height, this uses two lines,
			/* must clear NEXT line in x positions that where not used by
			/* double height, something might be there.
			*/
			y++;/* scip next line,
			/* it would overwrite the lower half of double height */
			for(i = 0; i < TXT_COLUMNS; i++)
				{
				if(dheight_xpositions_background[i] != -1)
									/* was not double height here */
					{
 					/* erase position */
 					s3txtputmsg(\
					posx[p] + (i * charwidth),\
					posy[p] + (y * charheight),\
					BLACK, dheight_xpositions_background[i], " ", 0);
					/* print space in correct background color */
					}
				}
			}/* end double hight erase */
		}/*end for each line*/
	}/*end for each page*/
/* print the macro clickbar in case it was not there :) */
if(macro_flag == MACRO_OFF)
	{
	print_macro_status(MACRO_A, MACRO_OFF);/* macro a used as dummy */
	}
return(1);
}

updctrl(b,v)/*update control port, bit, bit value*/
int b,v;
{
int p;
char iic_data[3];

return(1);
p = 1 << b;
if(! iicrcv(69, 1, iic_data) )return(0); /* dev addr. 40h, (sub 2) * 2, read */
if(v == 1)iic_data[0] = iic_data[0] | p;
if(v == 0)iic_data[0] = iic_data[0] & (255 - p);
if(! iicsnd(68, 1, iic_data) )return(0);
return(1);
}

getsupa(m,s,p)
/*get subpage (s) and page (p) in cct ram m*/
int m;
int *s;
int *p;
{
int a;
char iic_data[12];
if(! pgetstr(0, 25, m, 10, iic_data))return(0);
a = iic_data[2] & 15;
a += (iic_data[3] & 7) * 10;
a += (iic_data[4] & 15) * 100;
a += (iic_data[5] & 3) * 1000;
if(a > SUBMAX)a = 0;
*s = a;
a = iic_data[0] & 15; /*page units*/
a += (iic_data[1] & 15) * 10;/*page tenth*/
a += (iic_data[8] & 7) * 100;/*magazine*/
*p = a;
return(1);
}

/*clniicd(char iic_data[], int length)
/*replace 0 characters in iic_data with ' '*/
/*{
/*int i;
/*for(i = 0; i < length; i++)
/*	{
/*	if(iic_data[i] == 0)iic_data[i] = ' ';
/*	}
/* iicdat[i] = 0;/* string terminator */
/*return(1);
/*}
*/

tcls(m)/*clear display memory m*/
int m;
{
char iic_data[5];
if(!tputcu(0, 0, m))return(0);
iic_data[0] = 8;/*r8*/
iic_data[1] = 8 + m;
if(!iicsnd(34, 2 ,iic_data))return(0);
delay(200);/*delay for saa to execute cls command*/
return(1);
}

arrived(int saamem)/*returns 1 if page has arrived in memory m, else 0*/
{
int a, b;
int pblf, notfound;
char iic_data[20];
iic_data[0] = 8;/*r8*/
iic_data[1] = saamem;/*memory*/
iic_data[2] = 25;/*line*/
iic_data[3] = 0;/*column, status is here*/
if(!iicsnd(34, 4, iic_data))return(0);
if(!iicrcv(35, 10, iic_data))return(0);
/*pblf bit in byte 9 bit 5: nach seite wird gesucht*/
if(iic_data[9] & 0x20)	pblf = 1;
else	pblf = 0;
/*notfound bit in byte 8 bit 5: seite wurde gefunden*/
if(iic_data[8] & 0x20)	notfound = 1;
else	notfound = 0;
/*printf("\nbyte8 = %d  byte9 = %d  pblf = %d  notfound = %d",\
/*iic_data[8], iic_data[9], pblf, notfound);
/*do not know if notfound bit is set always by acqpag*/
if(pblf)return(0);/*not found*/
return(1);/*found*/
}

get_integer_value_pointed_at(int dspmem, int x, int y,\
int *selected_item, int *selected_item_tail)
/* get value of selected_item at his position,
/* move back to first space to find start of field,
/* if non digits in field return with value set to -1*/
{
int ptr;
char c;
char temp[42];
char iic_data[42];
int i;
int tail_flag;/*set if '/' detected in like 130/1*/
tail_flag = 0;/*field has no tail*/
/*read in the whole line*/
if(! pgetstr(0, y, dspmem, 40, iic_data) )return(0);/*error*/
/*clniicd(iic_data, 40);/*get rid of 0 characters in iic_data*/
/*find starting point of digits in iic_data*/
/*return these values if error or not found*/
*selected_item = -1;
*selected_item_tail = -1;
/*check if on digit, if not return ok with value -1*/
if(! isdigit(iic_data[x]) )
	{
	/*not a valid field*/
	if(iic_data[x] == '?')/* on '?' */
		{
		*selected_item = REVEAL_CONCEAL_TOGGLE;
		}
	return(1);/*ok*/
	}
ptr = x;/*start from here towards zero*/
while(1)/*scan iic_data*/
	{
	c = iic_data[ptr];
	if( isdigit(c) )
		{
		if(ptr == 0)break;/*start of line, valid, pointing at first digit*/
		ptr--;
		continue;
		}
	else
		{
		ptr++;
		break;/*space found, valid, pointing at first digit*/
		}
	}/*end while scan iic_data*/
/*sprintf(temp,"starting point found in iic_data is %d", ptr);
/*s3putmsg(0, 100, YELLOW, RED, temp, 1);
*/
/* left search for digits was ok, ptr points to first digit of field*/
/* copy from start of digits in iic_data to temp until space or end of line,
/* found, nondigit is error*/
while(1)
	{
	i = 0;/*ptr in temp*/
	while(1)/*copy digits to temp*/
		{
		c = iic_data[ptr];
		if( isdigit(c) )/*only copy digits*/
			{
			temp[i] = c;/*copy*/
			i++;
			if(ptr == 39)/*end of line, valid*/
				{
				temp[i] = 0;/*terminating zero*/
				break;
				}
			ptr++;
			continue;
			}
		else /*termination of digits field*/
			{
			temp[i] = 0;/*terminating zero*/
			break;
			}
		} /*end while copy digits to temp*/
	/*ptr points to last valid digit*/
	if(! tail_flag)
		{
		sscanf(temp, "%d", selected_item);/*read in value*/
		}
	else/*subpage*/
		{
		sscanf(temp, "%d", selected_item_tail);/* already a pointer! */
												/*read in value*/
		tail_flag = 0;
		return(1);
		}
	if( c == '/')
		{
		ptr++;
		tail_flag = 1;
		continue;/*get subpage*/
		}
	else return(1);/*no subpage, value set*/
	}/*end while page and subpage fields*/
return(1);/*ok*/
}/*end function get_integer_value_pointed_at*/

getmoney(descr,x,y,units,fract,me,le,ul,ll,fl,acf,ctfl,rangecheck)
/*get money value*/
/*description,hpos,vpos,units,fract,cctram,length,upper limit, lower limit,
/*fract length,array clear flag, check textfield flag*/
/*return error if outside limit*/
/*return error if fraction length incorrect (normally 2)*/
/*return error if descriptor incorrect*/
char descr[];/*descriptor, text found before value in line*/
float ul,ll;
int x,y,me,le,fl;
int acf;/*array clear flag*/
int *units,*fract;
int ctfl;
int rangecheck;
{
static char iic_data[42];
static char ar[42];/*must be preserved between calls to getmoney*/
static char aa[42];/*must be preserved between calls to getmoney*/

char des[42];/*descr is copied here to avoid changing descr*/
char ac[42];/*text in numeric field ("closed" or "gesloten")*/
int i,f,ffl;/*fraction flag*/
int lsf;/*leading spaces flag*/
char c;
char temp[80];
int not;/*notation, 0=Dutch, 1=others*/
int a;

*units=0;
*fract=0;
/*copy any valid characters from txt screen to acq array.
/*If signal is bad sometimes characters appear at some position then at next
/*time requested page comes by at other position.
/*characters that cannot be read appear as spaces.
/*All characters that are not spaces but valid digits or '.' or ',' are
/*collected in array ar, which is then read.
/*If no error in this read then data is used.
/*This is faster then waiting untill a complete correct field is received*/
/*For the discriptor the same procedure is used using array aa[], and
/*converting '_' to ' ' and lower to uppercase*/
/*A test is made if the descriptor is present before the value.
/*Finally the data is checked against the upper and lower limit*/

/*if first time read, wait a while to make sure cls was finished and the whole
/*screen has been updated*/
if(!pgetstr(0, y, me, 40, iic_data))return(0);/*read in the whole line*/
/*accumulate descriptor, and convert to uppercase*/
for(i = 0; i < x; i++)/*copy any characters from txt screen to array aa*/
	{
	if(acf)aa[i] = ' ';/*clear position on first read, calling routine
				/*must set acf*/
	c = iic_data[i];/*start at beginning of line*/
	if(c != ' ')aa[i] = toupper(c);/*copy anything that is no space and
					/*convert it to uppercase.
					/*teletext is not consistent in using
					/*cases, for example:
					/*Slot (goud) and slot (Dow)*/
	}
aa[i] = 0;/*string terminator*/
/*accumulate value*/
for(i = 0; i < le; i++)/*copy any valid data from txt screen to array ar*/
	{
	if(acf)
		{
		ar[i] = ' ';/*clear position on first read, calling routine
	   	ac[i] = ' ';	/*must reset acf*/
	    }
	c = iic_data[i + x];/*start at x position of value*/
	if((c == ',') || (c == '.') || (isdigit(c)))ar[i] = c;
	if(isalpha(c))ac[i] = c;/*in case text 'closed' 'gesloten'etc*/
	}
iic_data[i] = 0;/*string termination*/
ar[i] = 0;/*string termination for num field*/
ac[i] = 0;/*string termination for text field*/
clrstl(me);/*clear status line*/
sprintf(temp,"%s%s",aa,ar);/*print collected data*/
tputstr(0,24,me,temp);/*print collected data*/
/*check if text 'closed' or 'gesloten' in place of value*/
if((zindex(ac,"closed") != -1) || (zindex(ac,"gesloten") != -1))
	{
	(*units) = -1;/*negative units indicates special condition:
			/* -1 is closed, no trade due to holiday or others*/
	return(1);/*but data read correctly*/
	}
/*copy descriptor in local variable because it is going to be modified, else
/*fscanf will split it at the first space at the next read*/
strcpy(des,descr);
if(ctfl)/*if check text field flag*/
	{
	a=testdescriptor(aa,des,me,acf);/*does some printing!*/
	}
if(acf)printf("\n");/*CR LF if first time else previous line overwritten*/
printf("                                         \r");/*clear line 40 chars*/
printf("collected data: %s%s\r",aa,ar);/*print on the same line again and again*/
if(a == 0)return(0);/*after previous print,  error, descriptor incorrect*/
/*determine notation, teletext sometimes uses ',' and sometimes uses '.' as
/*unit separator for dow jones*/
not=0;/*default Dutch*/
if(fl == 0)/*no fraction*/
	{
	if(zindex(ar,",") != -1)not=1;/*a comma in units cannot be Dutch*/
	}
if(fl != 0)/*fraction present*/
	{
	if(zindex(ar,",") < zindex(ar,"."))not=1;
		/*no comma (-1) or comma before fullstop is not Dutch*/
	}
/*all others are Dutch notation*/
lsf=1;/*the leading space flag*/
ffl=0;/*fraction follows flag*/
f=0;/*length of fraction read*/
for(i=0;i<le;i++)
	{
	c=ar[i];
	if(lsf)	if(c == ' ')continue;/*scip leading space*/
	else lsf=0;/*reset leading space flag*/
	if(not == 0)/*Dutch notation*/
		{
		if(c == '.')continue;/*Dutch thousand separator, ignore*/
		if(c == ',')/*fract folllows Dutch notation*/
			{
			ffl=1;/*fract flag*/
			continue;
			}
		}
	if(not == 1)/*American and other notation*/
		{
		if(c == ',')continue;/*thousand separator, ignore*/
		if(c == '.')/*fraction follows*/
			{
			ffl=1;/*fract flag*/
			continue;
			}
		}
	if(!isdigit(c))break;
	c -= '0';/*ASCII to num*/
	if(!ffl)/*units*/
		{
		*units=10 * (*units) + c;
		}
	else/*fract*/
		{
		*fract=10 * (*fract) + c;
		f++;/*length of fraction read*/
		}
	}
if(f != fl)return(0);/*incorrect fraction length*/
if(rangecheck)
	{
	if(*units > ul)return(0);/*range error*/
	if(*units < ll)return(0);/*range error*/
	}
return(1);/*ok*/
}

getclk(int x, int y, int *ph, int *pm, int me)
	/*get the clock into hour and minute me=cctmem*/
{
char iic_data[40];
int h, m;
if(! pgetstr(x, y, me, 8, iic_data) )return(0);
if(! isdigit(iic_data[0]) )return(0);
h = iic_data[0] - 48;
if(! isdigit(iic_data[1]) )return(0);
h = h * 10 + (iic_data[1] - 48);
if(iic_data[2] != '.')return(0);
if(! isdigit(iic_data[3]) )return(0);
m = iic_data[3] - 48;
if(! isdigit(iic_data[4]) )return(0);
m = m * 10 + (iic_data[4] - 48);
*ph = h;
*pm = m;
return(1);
}

tputcu(x, y, m)
int x, y, m;
{
char iic_data[10];
iic_data[0] = 8;	/* r8 */
iic_data[1] = m;	/* cct mem */
iic_data[2] = y;
iic_data[3] = x;	/* column */
if(!iicsnd(34, 4, iic_data))return(0);
hpos[m] = x;
vpos[m] = y;
return(1);
}

pgetstr(int x, int y, int saamem, int length, char str[])
/* read cctram m row y column x length l in string str*/
{
str[0] = 8;/*r8*/
str[1] = saamem;/*memory*/
str[2] = y;/*start line*/
str[3] = x;/*start column*/
if(!iicsnd(34, 4, str))return(0);
if(!iicrcv(35, length, str))return(0);
return(1);
}

tputch(int m, char c)/*put character c at present cupos in dspmem m*/
{
char iic_data[10];
iic_data[0] = 11;/*r11*/
iic_data[1] = c;
if(!iicsnd(34, 2, iic_data))return(0);
return(1);
}

tputpage(int m, char s[])
/* puts string s into cctram m */
{
char iic_data[1024];
int i;
iic_data[0] = 8;/*r8*/
iic_data[1] = m;/*memory*/
iic_data[2] = 0;/* y start line*/
iic_data[3] = 0;/* x start column*/
if(!iicsnd(34, 4, iic_data))return(0);
for(i = 0; i < PAGLEN + 1; i++)
	{
	iic_data[i + 1] = s[i];
	}
iic_data[0] = 11;/*r11*/
if(!iicsnd(34, i + 1, iic_data))return(0);
return(1);
}

tputstr(int x, int y, int m, char s[])
/*puts string s into column x,line y,cctram m*/
{
char iic_data[1024];
int i;
iic_data[0] = 8;/*r8*/
iic_data[1] = m;/*memory*/
iic_data[2] = y;/*start line*/
iic_data[3] = x;/*start column*/
if(!iicsnd(34, 4, iic_data))return(0);
for(i = 0; i < PAGLEN + 1; i++)
	{
	if(s[i] == 0)break;
	iic_data[i + 1] = s[i];
	}
iic_data[0] = 11;/*r11*/
if(!iicsnd(34, i + 1, iic_data))return(0);
return(1);
}

acqpag(int p, int t, int ba, int ac, int mo, int hl)
/*(page,time,bank,acqcir,smode,hold)
/*reads requested page in cct ram
*/
{
int dcma,dcpt,dcpu,dcht,dchu,dcmt,dcmu;
char iic_data[20];
char temp[20];

tputstr(0, 0, 4 * ba + ac, "       ");/*the acq/bank screen, 7 spaces,
														/*clear the field*/
sprintf(temp,"%d",p);
tputstr(0, 0, 4 * ba + ac, temp);
if(mo == 1)/*time*/
	{
	sprintf(temp, "%d", t);
	tputstr(4, 0, 4 * ba + ac, temp);
	}
if(mo == 0)/*smode=page*/
	{
	dcma = 1;
	dcpt = 1;
	dcpu = 1;
	dcht = 0;
	dchu = 0;
	dcmt = 0;
	dcmu = 0;
	}
if(mo == 1)/*smode=time*/
	{
	dcma = 1;
	dcpt = 1;
	dcpu = 1;
	dcht = 1;
	dchu = 1;
	dcmt = 1;
	dcmu = 1;
	}
iic_data[0] = 2;/*r2*/
iic_data[1] = 64 * ba + 16 * ac;/*64*bank+16*acq.circuit+start column (0)*/
iic_data[2] = 16 * dcma + 8 * (1 - hl) + ((p / 100) & 7);/*hundreds,hold flag*/
iic_data[3] = 16 * dcpt + (((p % 100) / 10) & 15);/*tenth*/
iic_data[4] = 16 * dcpu + ((p % 10) & 15);/*units*/
iic_data[5] = 16 * dcht + ((t / 1000) & 3);/*ht*/
iic_data[6] = 16 * dchu + (((t % 1000) / 100) & 15);/*hu*/
iic_data[7] = 16 * dcmt + (((t % 100) / 10) & 7);/*mt*/
iic_data[8] = 16 * dcmu + ((t % 10) & 15);/*minutes units*/
if(!iicsnd(34, 9, iic_data))return(0);
delay(200);/*wait for saa to set flags in line 25 (pblf and notfound),
			/*else arrived will report page found at high iicbus speeds*/
last_active_bank = ba;/* this is now the active bank */
return(1);
}

signstat(int *s)
/*reads cct r11b, and determines from that the signal quality:
/*bit 0: video signal quality, 0=bad, 1=good
/*bit 1: text signal quality, 0=bad 1=good
/*bit 2 to bit 5: rom verion
*/
{
char iic_data[25];
iic_data[0] = 0;/*select r0*/
iic_data[1] = 1;/* bit 0=0 selects r11b*/
if(!iicsnd(0x22, 2, iic_data))
	{
	return(0);/*write error*/
	}
iic_data[0] = 1;/*r1*/
iic_data[1] = 4;/*bit 2=TCS (composite syc i.p.v. indirect sync)*/
if(!iicsnd(0x22, 2, iic_data))
	{
	return(0);
	}
iic_data[0] = 11;/*select r11*/
if(! iicrcv(0x23, 1, iic_data) )
	{
	return(0);/*read error*/
	}
*s = iic_data[0];/*the contents of r11b*/
iic_data[0] = 0;/*select r0*/
iic_data[1] = 0;/*bit 0=0 selects r11*/
if(!iicsnd(0x22, 2, iic_data))
	{
	return(0);/*write error*/
	}
return(1);/*ok*/
}

savepag(int m)/*get page from cct ram into iic_data[] and save it on disk*/
{
char iic_data[1024];
if(!pgetstr(0, 0, m, PAGLEN, iic_data))return(0);/*read cct pag m into iic_data*/
/*save array iic_data[0] to iic_data[PAGLEN-1] in file on disk*/
if(! svpag(iic_data))return(0);
/*note that this is different from loadpag(), where iic_data[0]=r11*/
return(1);
}

ldpag(char s[], char iic_data[])/*read page from disk into array iic_data[]*/
{
int i;
FILE *infile;
infile = fopen(s,"rb");
if(! infile)return(0);
for(i = 0; i < PAGLEN;i++)
	{
	iic_data[i+1] = getc(infile);
	/*iic_data[0] reserved for reg 11 command in loadpag*/
	}
fclose(infile);
return(1);
}

writeram(int channel, int page, int subpage, char data[])
{
char temp[20];
sprintf(temp,"%3d%03d%1d",page,subpage);
if(install(temp, data, PAGLEN) )return(1);
printf("\nCannot install field %s in ram, out of memory\n", temp);
return(0);
}

delay(int milliseconds)/*software delay, replaces function delay*/
{
int i;
long li;
for(i =0; i < milliseconds; i++)
	{
	for(li = 0; li <  10000; li++)/*calibrate for processor and system*/
		{
		;
		}
	}
}

user_echo(int x, int y, int fg, int bg, char buffer[], int max_length)
{
char temp[2];
int ptr;

ptr = 0;
/* print characters from buffer until 0 found */
temp[1] = 0;/*string termination*/
while(1)
   	{
   	if(ptr >= max_length)break;
 	temp[0] = buffer[ptr];
   	if(temp[0] == 0)break;/*end of buffer*/
   	s3putmsg(x + ptr * charwidth, y, fg, bg, temp, 1);
   	ptr++;
   	}
/* print spaces until max_length, this overwrites previous characters
/* after backspace */
while(1)
   	{
   	if(ptr >= max_length)break;
   	temp[0] = ' ';/*overwrite with space*/
   	s3putmsg(x + ptr * charwidth, y, fg, bg, temp, 1);
	ptr++;
	}
return(1);
}/*end function user_echo*/

copy_down(char string[], int start, int space, int maxlen)
/*copy up string str from start space characters*/
{
int ptr;
char c;
/* first start + space to start, then start + space + 1 to start + 1, etc.*/
if(start < 0)return(0);/*boundary*/
ptr = start;
while(1)
	{
	if( (ptr + space) > maxlen)return(0);/*boundary*/
	c = string[ptr + space];
	string[ptr] = c;/*copy down*/
	if(c == 0)return(1);/*array termination also copied, ready*/
	ptr++;
	}
}/*end function copy_down*/

copy_up(char string[], int start, int space, int maxlen)
/*copy up string str from start space characters*/
{
int ptr;
char c;
int top;
/*find end of string*/
top = 0;
while(1)
	{
	c = string[top];
	if(c == 0)break;/*found end*/
	top++;
	if(top > maxlen)return(0);/*error, top not found*/
	}
ptr = top;
/* first top to top + space, then top - 1 to top + space - 1, etc. */
while(1)
	{
	/*test for array boundary*/
	if( (ptr < 0) || (ptr > (maxlen - space) ) )return(0);/*error*/
	c = string[ptr];
	string[ptr + space] = c;/* copy up starting at top*/
	if(ptr == start)return(1);/*ready*/
	ptr--;
	}
return(1);/*ok*/
}/*end function copy_up*/


move_softcursor(int x, int y)
{
unpaint_softcursor();
paint_softcursor(x, y);
return(1);
}

unpaint_softcursor()
{
int pixel;
int dx, dy;

for(dx = 0; dx < charwidth; dx++)
	{
	for(dy = 0; dy < charheight; dy++)
		{
		if(dy == charheight - 1)
			{
			pixel = oldpixels[dx][dy];
			s3plot(oldx + dx, oldy + dy, pixel);
			}
		}
	}
return(1);
}

paint_softcursor(int x, int y)
{
int pixel;
int dx, dy;

for(dx = 0; dx < charwidth; dx++)
	{
	for(dy = 0; dy < charheight; dy++)
		{
		pixel = s3getpixel(x + dx, y + dy);
		oldpixels[dx][dy] = pixel;
		if(dy == charheight - 1)
			{
			pixel = WHITE;
/*			if(dy == charheight - 2)pixel = WHITE;/* underline */
			s3plot(x + dx, y + dy, pixel);
			}
		oldx = x;	
		oldy = y;
		}
	}
return(1);
}

xget_input_string_with_echo(int x, int y, int max_length,\
		int selected_display_page, char buffer[], int key)
{
static int cursor_position;
int a;
char c, d;
int ptr;
static char temp[80];
int fg,bg;
static int first_time_flag;

fg = WHITE;
bg = RED;
x *= charwidth;
y *= charheight;
if(selected_display_page == 1)
	{
	x += 320;	/* 640 / 2 */
	}
if(selected_display_page == 2)
	{
	y += 240;	/* 480 / 2 */
	}
if(selected_display_page == 3)
	{
	x += 320;
	y += 240;
	}
/* the editor, mode is always insert */
if(first_time_flag || mouse_click)
	{
	buffer[0] = 0;/* mark empty */
	cursor_position = 0;
	}
if( (key == 13) || (key == 10) )/*ENTER pressed*/
	{
	/*set these for next time, set to zero on first start of program,
	/* statics! */
	first_time_flag = 1;/* reset */
	return(EOF);
	}
first_time_flag = 0;
if(key == BACKSPACE)
	{
	if(cursor_position > 0)/* erase character left of cursor in buffer */
		{
		cursor_position--;
		/*copy down the buffer*/
		copy_down(buffer, cursor_position, 1, max_length);
		}
	}
if(key == DELETE)
	{
	/* erase character at cursor position */
	copy_down(buffer, cursor_position, 1, max_length);
	}
if(key ==  CU_LEFT)
	{
	if(cursor_position > 0) cursor_position--;
	}
if(key == CU_RIGHT)
	{
	if(cursor_position < strlen(buffer) )cursor_position++;
	}
if( (key > ' ') && (key < 127) )/*valid character*/
	{
	/* a valid character, insert it at cursor position */
	/* test for space */
	if( (cursor_position < max_length) && (strlen(buffer) < max_length) )
		{
		/* make space, copy up the buffer 1 */
		copy_up(buffer, cursor_position, 1, 79);
		/* insert character at cursor position */
		buffer[cursor_position] = key;
		cursor_position++;
		/* the buffer is now longer, if it extends beyond max_length,
		/* then that part must be truncated */
		}	
	else  /* no space */
		{
		/* signal user */
		sbeep(22, 2000);/* 22 milliseconds at 2000 Hz */
		/* do nothing if no space */
		}
	}/* end valid character */
/* ignore other keys */
/*echo to user the buffer*/
user_echo(x, y, fg, bg, buffer, max_length);
move_softcursor(x + cursor_position * charwidth, y);
mouse_click = 0;
return(1);
}/* end function xget_input_string_with_echo */

load_page(char name[], int selected_display_page, int dspmem_offsett)
/* reads page 'name' from disk into saamem corresponding to
/* selected_display_page and dspmem_offsett*/
/* hold must have been set prior to this, else page will be overwritten
*/
{
FILE *infile;
int i;
int decoder;
int saamem;
char filename[40];
char path[40];
char temp[80];
char iic_data[1024];

strcpy(temp, name);
if(zindex(temp, ".") == -1)strcat(temp,".txt");
get_path_name("database_files_dir", path);
sprintf(filename, "%s%s", path, temp);
/*calculate the saamem*/
saamem = selected_display_page + dspmem_offsett;
/*data to saamem*/
iic_data[0] = 8;		/* r8 */
iic_data[1] = saamem;	/* memory */
iic_data[2] = 0;		/* start line */
iic_data[3] = 0;		/* start column */
if(!iicsnd(34, 4, iic_data))return(0);
/*load iic_data[] from disk*/
infile = fopen(filename, "rb");
if(! infile)
	{
	print_in_display(selected_display_page, "NOT FOUND");
	return(1);
	}
for(i = 0; i < PAGLEN; i++)
	{
	iic_data[i + 1] = getc(infile);
	/*iic_data[0] reserved for reg 11 command in loadpag*/
	}
fclose(infile);
iic_data[0] = 11; /* r11, data in iic_data[1] to iic_data[PAGLEN+1]*/
/*write array iic_data[] to cct ram*/
if(!iicsnd(34, PAGLEN + 1, iic_data))return(0);
sprintf(temp, "%s loaded", name);
print_in_display(selected_display_page, temp);
return(1);
}/*end function load_page*/

save_page(char name[], int selected_display_page, int dspmem_offsett)
/* writes  saamem corresponding to selected_display_page and dspmem_offsett
/* to disk.
*/
{
FILE *outfile;
char temp[80];
int i;
int decoder;
int saamem;
char c;
char filename[40];
char path[80];
char iic_data[1024];

strcpy(temp, name);
if(zindex(temp, ".") == -1)strcat(temp,".txt");
get_path_name("database_files_dir", path);
sprintf(filename, "%s%s", path, temp);
/* calculate the saamem */
saamem = selected_display_page + dspmem_offsett;
/* test if exists */
outfile = fopen(filename, "rb");
if(outfile)/*exists*/
	{
	fclose(outfile);
	print_in_display(selected_display_page,\
	"EXISTS, overwrite (y/n)?                 ");
/*
	get_input_string_with_echo(24, 25, 1, selected_display_page, temp);
	c = toupper(temp[0]);
	if(c == 'N')
		{
		print_in_display(selected_display_page, "Cancelled");
		return(1);
		}
*/
	}
/* create (new) */
outfile = fopen(filename, "wb");
if(! outfile)
	{
	print_in_display(selected_display_page, "CANNOT SAVE");
	return(1);
	}
/* get data from saamem */
if(!pgetstr(0, 0, saamem, PAGLEN, iic_data) )return(0);/*hardware error*/
/* write iic_data to disk */
for(i = 0; i < PAGLEN; i++)
	{
	putc(iic_data[i], outfile);
	}
fclose(outfile);
/* report to user */
sprintf(temp, "%s saved", name);
print_in_display(selected_display_page, temp);
return(1);
}/* end function save_page */

int put_display_page_in_ram(int selected_display_page, int dspmem_offsett)
/*saves selected display page in RAM*/
{
char temp[20];
char iic_data[1024];
/*x, y, dspmem(saa), lenght, char array*/
if(! pgetstr(0, 0, selected_display_page + dspmem_offsett, PAGLEN, iic_data) )
	{
	return(0);
	}
sprintf(temp, "SDP%3d", selected_display_page);
/*if(! install(temp, iic_data, PAGLEN) )*/
if(! data_to_ram(temp, iic_data) )
	{
	print_in_display(selected_display_page, "NO SPACE TO STORE PAGE");
	return(0);
	}
return(1);
}

int get_display_page_from_ram(int selected_display_page, int dspmem_offsett,\
int _hold_page_flag[])
{
char temp[20];
char iic_data[1024];
/*set hold for the corresponding acquisition circuit*/
_hold_page_flag[selected_display_page] = 1;
/*send the hold*/
if(! hold_page(selected_display_page, dspmem_offsett) )return(0);
sprintf(temp,  "SDP%3d", selected_display_page);
if(! data_from_ram(temp, iic_data) )
	{
	print_in_display(selected_display_page, "NO PREVIOUS PAGE");
	return(1);
	}
/*x, y, dspmem(saa), data*/
/* the difference between tputpage and tputstr is that tputpage may contain 0 */
if(! tputpage(selected_display_page + dspmem_offsett, iic_data) )
	{
	return(0);	/*hardware error*/
	}
print_in_display(selected_display_page, "PREVIOUS PAGE");
return(1);
}


